//+------------------------------------------------------------------+
//|                                                        VA_MA.mq4 |
//|                               Copyright  2014, Gehtsoft USA LLC |
//|                                            http://fxcodebase.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2014, Gehtsoft USA LLC"
#property link      "http://fxcodebase.com"

#property indicator_chart_window
#property indicator_buffers 1
#property indicator_color1 Yellow

extern string TimeFrame       = "Current time frame";
extern int Length=15;
extern bool Use_Double_Smooth=true;
extern int Price=0;    // Applied price
                       // 0 - Close
                       // 1 - Open
                       // 2 - High
                       // 3 - Low
                       // 4 - Median
                       // 5 - Typical
                       // 6 - Weighted  

double VAMA[];
double Raw[];
string indicatorFileName;
bool   returnBars;
int    timeFrame;

int init()
{
 IndicatorShortName("Velocity and Accerlation Moving Average");
 IndicatorBuffers(2);
 SetIndexBuffer(0,VAMA);
 SetIndexBuffer(1,Raw);
         timeFrame         = stringToTimeFrame(TimeFrame);
         indicatorFileName = WindowExpertName();
         returnBars        = TimeFrame=="returnBars";

 return(0);
}

int deinit()
{

 return(0);
}

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { VAMA[0] = MathMin(limit+1,Bars-1); return(0); }

   if (timeFrame != Period())
   {
      limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
      for(int i=limit; i>=0; i--)
      {
         int y = iBarShift(NULL,timeFrame,Time[i]);
            VAMA[i] = iCustom(NULL,timeFrame,indicatorFileName,"",Length,Use_Double_Smooth,Price,0,y);
      }            
      return(0);
   }      

 int pos;
 double vel, acc, aaa;
 double MA0, MA4, MA8, MA12;
 pos=limit;
 while(pos>=0)
 {
  MA0=iMA(NULL, 0, Length, 0, MODE_EMA, Price, pos);
  MA4=iMA(NULL, 0, Length, 0, MODE_EMA, Price, pos+Length/4);
  MA8=iMA(NULL, 0, Length, 0, MODE_EMA, Price, pos+Length/8);
  MA12=iMA(NULL, 0, Length, 0, MODE_EMA, Price, pos+Length/12);
  vel=MA0-MA4;
  acc=MA0-2.*MA4+MA8;
  aaa=MA0-3.*MA4+3.*MA8-MA12;
  Raw[pos]=MA0+vel+acc/2.+aaa/6.;

  pos--;
 } 
 
 pos=limit;
 while(pos>=0)
 {
  if (Use_Double_Smooth)
  {
   VAMA[pos]=iMAOnArray(Raw, 0, Length/4, 0, MODE_EMA, pos);
  }
  else
  {
   VAMA[pos]=Raw[pos];
  }

  pos--;
 }
   
 return(0);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}